"""control module"""
import datetime
import json
import logging
import os
import shutil
import threading
import time

import RPi.GPIO as GPIO

class Control(threading.Thread):
    """Control class"""
    def __init__(self, configfile):
        threading.Thread.__init__(self)
        self.run_condition = True
        self.config_file = configfile
        self.config = None
        self.log = logging.getLogger()
        self.trigger_read_config = True
        self.water_state = []

    def reload_config(self):
        """Reload the config"""
        self.log.info("Reloading configuration triggered")
        self.trigger_read_config = True

    def load_config(self):
        """load the config"""
        try:
            with open(self.config_file, "r", encoding="UTF-8") as handle:
                self.config = json.load(handle)
        except FileNotFoundError:
            # create default config
            os.makedirs(os.path.dirname(self.config_file), exist_ok=True)
            shutil.copyfile("config/config.json", self.config_file)
            with open(self.config_file, "r", encoding="UTF-8") as handle:
                self.config = json.load(handle)
        for _ in range(len(self.config['water'])):
            self.water_state.append(False)
        # Configure all water pins
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)
        for entry in self.config['water']:
            pin = int(entry['pin'])
            GPIO.setup(pin, GPIO.OUT)
            GPIO.output(pin, 1)

    def run(self):
        while self.run_condition:
            if self.trigger_read_config:
                self.trigger_read_config = False
                self.load_config()

            # handle water entries
            water = self.config['water']
            water_index = 0
            for entry in water:
                now = datetime.datetime.now()
                if entry['autostate']:
                    idx = 0
                    if int(now.hour) >= 12:
                        idx = 1
                    on_time_pattern = entry['times'][idx]['on_time']
                    on_time_pattern = on_time_pattern.split(':')
                    on_time = now.replace(hour=int(on_time_pattern[0]),
                                          minute=int(on_time_pattern[1]),
                                          second=0,
                                          microsecond=0)
                    off_time_pattern = entry['times'][idx]['off_time']
                    off_time_pattern = off_time_pattern.split(':')
                    off_time = now.replace(hour=int(off_time_pattern[0]),
                                           minute=int(off_time_pattern[1]),
                                           second=0,
                                           microsecond=0)
                    pin = int(entry['pin'])

                    if now > on_time and now <= off_time and not self.water_state[water_index]:
                        GPIO.output(pin, 0)
                        self.water_state[water_index] = True
                        self.log.info("water on")
                    elif now > off_time and self.water_state[water_index]:
                        GPIO.output(pin, 1)
                        self.water_state[water_index] = False
                        self.log.info("water off")
                water_index += 1
            time.sleep(1)


    def stop(self):
        """Stop execution"""
        self.run_condition = False
        self.join()

    def get_current_water_state(self, ident: int):
        """Get water state"""
        if ident > 0 and ident <= len(self.water_state):
            return self.water_state[ident -1]
        return None

    def set_water_state(self, ident: str):
        """Set water state"""
        ident = int(ident)
        if ident > 0 and ident <= len(self.water_state):
            pin = int(self.config['water'][ident - 1]['pin'])
            self.water_state[ident - 1] = True
            self.log.info("water on by button")
            GPIO.output(pin, 0)

    def clear_water_state(self, ident: str):
        """Clear water state"""
        ident = int(ident)
        if ident > 0 and ident <= len(self.water_state):
            pin = int(self.config['water'][ident - 1]['pin'])
            self.water_state[ident - 1] = False
            self.log.info("water off by button")
            GPIO.output(pin, 1)
